import os
import typer
from rich import print as rprint
import yaml
import shutil

app = typer.Typer()

@app.command(name="project", help="Create a new project")
def create_project(name: str):
    rprint(f"[yellow bold]Creating new project: {name}[/yellow bold]")

    # TODO: support more templates
    base_dir = os.path.dirname(__file__)
    project_template = base_dir + '/templates/default'
    try:
        shutil.copytree(project_template, name) # Also creates the directory
    except Exception as e:
        rprint(f"[red bold]An error occurred setting up the project template:[/red bold] {str(e)}")

    # TODO: should be able to override default config via CLI options
    default_config = {
        'log_level': 'INFO',
        'input': {
            'video': {
                'enable': True,
                'uri': 'file:///home/your/path'
            },
            'address': { # address where the input component runs for the nng connections
                'host': 'localhost',
                'port': 1234
            },
        },
        "output": {
            'video': {
                'enable': True,
                'uri': 'file:///home/your/path'
            },
            'address': { # address where the input component runs for the nng connections
                'host': 'localhost',
                'port': 1237
            },
        },
        'worker': {
            'n_workers': 1,
        },
    }

    try:
        # TODO: override default config with user provided args
        new_config_file=open(f"{name}/config.yaml","w")
        yaml.dump(default_config, new_config_file)
        new_config_file.close()
        rprint("[yellow]Config file created.[/yellow]")
    except Exception as e:
        rprint(f"[red bold]An error occurred setting up the project config file:[/red bold] {str(e)}")



from rich import print as rprint
import yaml
import os
from pipeless_ai.core import Pipeless

def run_app(component: str):
    """
    The run command must be executed from the project dir.
    It will load all apps under the project's 'apps' directory.

    The component is 'input', 'output', 'worker' or 'all'
    """
    rprint(f"[yellow]Running project...[/yellow]")

    exec_dir = os.getcwd()
    config_file_path = os.path.join(exec_dir, 'config.yaml')

    rprint('Loading config.yaml...')
    config_file=open(config_file_path, "r")
    config = yaml.safe_load(config_file)
    config_file.close()
    rprint('[green]Config loaded[/green]')

    app_filename = 'app.py'
    app_path = os.path.join(exec_dir, app_filename)

    Pipeless(config, component, app_path)



from pipeless_ai.lib.app.app import PipelessApp

class App(PipelessApp):
    """
    Main application class.

    Pre-process, process and post-process hooks, if implemented,
    must return a RGB image.

    The context can be accessed and modified at any stage of the pipeline.
    You can use it to share data between stages or pipeline iterations (i.e
    between the processing of different frames)
    """

    # Hook to execute before the processing of the first image
    def before(self, ctx):
        pass

    # Hook to execute to pre-process each image
    def pre_process(self, frame, ctx):
        modified_frame = frame # Do something to the frame
        return modified_frame

    # Hook to execute to process each image
    def process(self, frame, ctx):
        modified_frame = frame # Do something to the frame
        return modified_frame

    # Hook to execute after processing each image
    def post_process(self, frame, ctx):
        modified_frame = frame # Do something to the frame
        return modified_frame

    # Hook to execute after the processing of the last image
    def after(self, ctx):
        pass



import typer
from .commands import create, run

app = typer.Typer()

app.add_typer(create.app, name="create", help="Create project resources")

@app.command('run', help='Execute the project')
def run_project(component: str = typer.Argument("all")):
    run.run_app(component)

if __name__ == "__main__":
    app()
